@@ -8,12 +8,13 @@ import unittest, sys, os.path
 
 this_dir = os.path.dirname(__file__)
 if this_dir not in sys.path:
-    sys.path.insert(0, this_dir) # needed for Py3
+    sys.path.insert(0, this_dir)  # needed for Py3
 
-from common_imports import etree, StringIO, BytesIO, _bytes, doctest
+from common_imports import etree, BytesIO, _bytes
 from common_imports import HelperTestCase, make_doctest, skipIf
 from common_imports import fileInTestDir, fileUrlInTestDir
 
+
 class ETreeDtdTestCase(HelperTestCase):
     def test_dtd(self):
         pass
@@ -34,13 +35,14 @@ class ETreeDtdTestCase(HelperTestCase):
     def test_dtd_parse_invalid(self):
         fromstring = etree.fromstring
         parser = etree.XMLParser(dtd_validation=True)
-        xml = _bytes('<!DOCTYPE b SYSTEM "%s"><b><a/></b>' % fileInTestDir("test.dtd"))
+        xml = _bytes('<!DOCTYPE b SYSTEM "%s"><b><a/></b>' %
+                     fileInTestDir("test.dtd"))
         self.assertRaises(etree.XMLSyntaxError,
                           fromstring, xml, parser=parser)
 
     def test_dtd_parse_file_not_found(self):
         fromstring = etree.fromstring
-        dtd_filename = fileInTestDir("__nosuch.dtd")
+        dtd_filename = fileUrlInTestDir("__nosuch.dtd")
         parser = etree.XMLParser(dtd_validation=True)
         xml = _bytes('<!DOCTYPE b SYSTEM "%s"><b><a/></b>' % dtd_filename)
         self.assertRaises(etree.XMLSyntaxError,
@@ -58,7 +60,8 @@ class ETreeDtdTestCase(HelperTestCase):
 
     def test_dtd_parse_valid(self):
         parser = etree.XMLParser(dtd_validation=True)
-        xml = '<!DOCTYPE a SYSTEM "%s"><a><b/></a>' % fileInTestDir("test.dtd")
+        xml = ('<!DOCTYPE a SYSTEM "%s"><a><b/></a>' %
+               fileUrlInTestDir("test.dtd"))
         root = etree.fromstring(xml, parser=parser)
 
     def test_dtd_parse_valid_file_url(self):
@@ -71,7 +74,7 @@ class ETreeDtdTestCase(HelperTestCase):
         parser = etree.XMLParser(dtd_validation=True)
         xml = '<!DOCTYPE a SYSTEM "test.dtd"><a><b/></a>'
         root = etree.fromstring(
-            xml, parser=parser, base_url=fileInTestDir("test.xml"))
+            xml, parser=parser, base_url=fileUrlInTestDir("test.xml"))
 
     def test_dtd_parse_valid_relative_file_url(self):
         parser = etree.XMLParser(dtd_validation=True)
@@ -114,6 +117,22 @@ class ETreeDtdTestCase(HelperTestCase):
         self.assertTrue(dtd)
         self.assertFalse(dtd.validate(root))
 
+    def test_dtd_invalid_duplicate_id(self):
+        root = etree.XML(_bytes('''
+        <a><b id="id1"/><b id="id2"/><b id="id1"/></a>
+        '''))
+        dtd = etree.DTD(BytesIO(_bytes("""
+        <!ELEMENT a (b*)>
+        <!ATTLIST b
+            id ID #REQUIRED
+        >
+        <!ELEMENT b EMPTY>
+        """)))
+        self.assertFalse(dtd.validate(root))
+        self.assertTrue(dtd.error_log)
+        self.assertTrue([error for error in dtd.error_log
+                         if 'id1' in error.message])
+
     def test_dtd_api_internal(self):
         root = etree.XML(_bytes('''
         <!DOCTYPE b SYSTEM "none" [
@@ -234,7 +253,7 @@ class ETreeDtdTestCase(HelperTestCase):
             attributes)
 
     def test_dtd_attrs(self):
-        dtd = etree.DTD(fileInTestDir("test.dtd"))
+        dtd = etree.DTD(fileUrlInTestDir("test.dtd"))
 
         # Test DTD.system_url attribute
         self.assertTrue(dtd.system_url.endswith("test.dtd"))
@@ -278,12 +297,37 @@ class ETreeDtdTestCase(HelperTestCase):
         parser = etree.XMLParser(dtd_validation=True)
         xml = '<!DOCTYPE a SYSTEM "test.dtd"><a><b/></a>'
         root = etree.fromstring(xml, parser=parser,
-                                base_url=fileInTestDir("test.xml"))
+                                base_url=fileUrlInTestDir("test.xml"))
 
         dtd = root.getroottree().docinfo.internalDTD
         self.assertEqual(dtd.name, "a")
         self.assertEqual(dtd.system_url, "test.dtd")
 
+    def test_declaration_escape_quote_pid(self):
+        # Standard allows quotes in systemliteral, but in that case
+        # systemliteral must be escaped with single quotes.
+        # See http://www.w3.org/TR/REC-xml/#sec-prolog-dtd.
+        root = etree.XML('''<!DOCTYPE a PUBLIC 'foo' '"'><a/>''')
+        doc = root.getroottree()
+        self.assertEqual(doc.docinfo.doctype,
+                         '''<!DOCTYPE a PUBLIC "foo" '"'>''')
+        self.assertEqual(etree.tostring(doc),
+                         _bytes('''<!DOCTYPE a PUBLIC "foo" '"'>\n<a/>'''))
+
+    def test_declaration_quote_withoutpid(self):
+        root = etree.XML('''<!DOCTYPE a SYSTEM '"'><a/>''')
+        doc = root.getroottree()
+        self.assertEqual(doc.docinfo.doctype, '''<!DOCTYPE a SYSTEM '"'>''')
+        self.assertEqual(etree.tostring(doc),
+                         _bytes('''<!DOCTYPE a SYSTEM '"'>\n<a/>'''))
+
+    def test_declaration_apos(self):
+        root = etree.XML('''<!DOCTYPE a SYSTEM "'"><a/>''')
+        doc = root.getroottree()
+        self.assertEqual(doc.docinfo.doctype, '''<!DOCTYPE a SYSTEM "'">''')
+        self.assertEqual(etree.tostring(doc),
+                         _bytes('''<!DOCTYPE a SYSTEM "'">\n<a/>'''))
+
 
 def test_suite():
     suite = unittest.TestSuite()
